var calculator = new function()
{
  var self = this;
  
  var value_entered = ''; // string
  var value_submitted = 0; // number
  var value_calculated = 0; // number
  var value_onscreen = 0; // number
  
  var operation = '';
  var operation_previous = ''; // backup of previous operation
  
  var memory = parseFloat(window.widget && widget.preferenceForKey('memory') || 0) || 0;
  var display_view = null;
  var operation_view = null;
  var float_buffer = '';
  var C_INT = 0;
  var C_FLOAT = '';
  var C_EXP = 0;

  // constants
  var C_MAXPOWER = 15; // max power before using scientific
  var C_MAXZEROS = 6; // max number of zeros, i.e. 0.00003
  var C_ROUNDING_THRESHOLD = 1e-14; // the amount to round down a number like 3.99999999
  
  // error messages
  var C_ERROR_SQRT = 'Cannot square root negatives';
  var C_ERROR_DIVIDEZERO = 'Cannot divide by zero';
  var C_ERROR_ZERODIVIDEZERO = 'Cannot divide by zero';
  var C_ERROR_INFINITE = 'Too big';
  
  var operations =
  {
    'divide': function()
    {
      if (value_submitted == 0)
      {
        if (value_calculated == 0)
        {
          self['clear']();
          throw C_ERROR_ZERODIVIDEZERO;
        }
        else
        {
          self['clear']();
          throw C_ERROR_DIVIDEZERO;
        }
      }
      else
      {
        value_calculated /= value_submitted;
        return null;
      }
    },
    'multiply': function()
    {
      value_calculated *= value_submitted;
      return null;
    },
    'plus': function()
    {
      value_calculated += value_submitted;
      return null;
    },
    'minus': function()
    {
      value_calculated -= value_submitted;
      return null;
    }
  }

  var handle_input = function(event)
  {
    var handler = event.target.getAttribute('handler');
    if( handler && self[handler] )
    {
      event.stopPropagation();
      event.preventDefault();
      self[handler](event);
    }
  }

  var init = function()
  {
    var cal = document.getElementById('calculator');
    if(cal)
    {
      cal.addEventListener('click', handle_input, true);
      display_view = document.getElementById('display');
      operation_view = document.getElementById('operation');
      updateDisplay(value_calculated);
    }
    // avoid text-selection:
    document.onmousemove = document.onmousedown = function(e)
    {
        e.preventDefault();
    };
  }

  var group_float = function(str)
  {
    var ret = '', pos = 0;
    while( str.length > 3 )
    {
      ret += str.slice(0,3) +' ';
      str = str.slice(3);
    }
    return ret + str;
  }

  var group_int = function(str)
  {
    var ret = '', pos = 0, l = 0;
    while( ( l = str.length ) > 3 )
    {
      ret = ' ' + str.slice( l - 3 ) + ret;
      str = str.slice( 0, l - 3 );
    }
    return str + ret;
  }
  
  function displayOperation(operation)
  {
    var offsets = {
      '' : 0,
      'plus'    : 4*33,
      'minus'   : 3*33,
      'multiply': 2*33,
      'divide'  : 1*33
      
    }
    document.getElementById('operation').style.backgroundPosition = '0 '+(offsets[operation] || 0) + 'px';
  }
  
  function updateDisplay(value)
  {    
    var num = 0;
    var str = "";
    
    switch (typeof value)
    {
      case 'number' :
        value_onscreen = value;
        try {
          str = numToStr(value);
        }
        catch (e)
        {
          value = null;
        }
        break;
      
      case 'string' :
        value_onscreen = strToNum(value);
        str = value;
        break;
    }
    
    if (value === null)
    {
      document.getElementById('error').style.visibility = 'visible';
      document.getElementById('digits').style.visibility = 'hidden';
    }
    else
    {
      document.getElementById('error').style.visibility = 'hidden';
      document.getElementById('digits').style.visibility = 'visible';
    }
    
    var ol = document.getElementsByTagName('ol')[0].getElementsByTagName('li');
    
    var offsets =
    {
      '9' : 33*1,
      '8' : 33*2,
      '7' : 33*3,
      '6' : 33*4,
      '5' : 33*5,
      '4' : 33*6,
      '3' : 33*7,
      '2' : 33*8,
      '1' : 33*9,
      '0' : 33*10,
      '-' : 33*11,
      'e' : 33*12
    }

    var point = document.getElementById('point');
    var c = str.length;
    
    for (var i=0; i<ol.length; i++)
    {
      c--;
      
      if (c>=0 && str[c]=='.')
      {
        point.style.backgroundPosition = (699-23*(str.length-c)) + 'px 165px';
        c--;
      }
      
      ol[ol.length-i-1].style.backgroundPosition = (c<0) ? ('0 0') : ('0 ' + (-offsets[str[c]]||0) + 'px');
    }
    
    if (str.indexOf('.') < 0)
    {
      point.style.backgroundPosition = '-10px 0';
    }

    
  };

  var set_float_buffer = function()
  {
    extractComponents();
    if( !C_EXP && C_FLOAT )
    {
      float_buffer = '.' + C_FLOAT;
    }
    else
    {
      float_buffer = '';
    }
  }
  
  function strToNum(str)
  {
    var num = parseFloat(str) || 0;
    return num;
  }

  function numToStr(num)
  {
    var absNum = num<0 ? -num : num;
    
    var max = Math.pow(10,C_MAXPOWER);
    var min = Math.pow(0.1,C_MAXZEROS);
    
    var i = 0;
    
    var exp, coe;

    if (!isFinite(num))
    {
      self['clear']();
      throw C_ERROR_INFINITE;
    }

    if (absNum < max && absNum > min || num==0)
    {
      // sometimes we get values like 4.99999999999
      // we want to round them up to 5
      // an example is 45/6/7/9*6*7*9 which turns out as 44.9999
      // or 43/6/7/9*6*7*9 which turns out as 43.0000000
      // C_ROUNDING_THRESHOLD is a very small number to check agains
      
      

      if ((1-absNum%1) < C_ROUNDING_THRESHOLD) // round up 44.999
      {
        return ''+Math.ceil(absNum)*(num<0?-1:1);
      }

      
  
      coe = (num+'');
      
          /*
      if (coe.indexOf('.')>=0)
      {
        for (i=coe.length-2; i>=0; i--) // we don't care about the last digit
        {
          if (coe.substr(i,1)!='9')
          {
            break;
          }
        }
      }
      if (coe.length-i > 5)
      {
        coe = coe.substr(0,i+1);
        coe = ''+(parseFloat(coe)+parseFloat('.'+('0000000000000000000'.substr(0,coe.length-coe.indexOf('.')-2))+'1'));
      }
      */
      
      // else trim off any zeros after a decimal and ship it off
      coe = coe.substr(0,C_MAXPOWER)
      while (coe.indexOf('.')>=0 && (coe[coe.length-1]=='0' || coe[coe.length-1]=='.'))
      {
        coe = coe.substr(0,coe.length-1);
      }
      return coe;
    }
    
    // best way to get the exponent is if it is in the
    // number's string, but that may not be the case
    exp = (num+'').indexOf('e');
    if (exp >=0 )
    {
      coe = (num+'').substr(0,exp);
      exp = parseFloat((num+'').substring(exp+1, (num+'').length));
    }
    else
    {
      if (absNum >= max) // big-number
      {
        // doing a loop is the only way to find
        // the exponent accurately, otherwise
        // you get rounding errors
        for (exp=1; exp<100; exp++)
        {
          if (num < Math.pow(10,exp))
          {
            exp--;
            break;
          }
        }
      }
      else // small number
      {
        exp = Math.floor(Math.log( absNum )/Math.LN10);
      }
      coe = num / Math.pow(10,exp);
    }
    
    coe = '' + coe;
    
    if (coe.length > C_MAXPOWER)
    {
      // count the number of nines
      for (i=0; i<coe.length; i++)
      {
        if (coe[i]!='9' && coe[i]!='.')
        {
          break;
        }
      }
      // avoid annoying rounding issue of 10e23 being 9.999e22
      if (i>C_MAXPOWER+1)
      {
        coe = '1';
        exp++;
      }
    
      // trim size of coeff
      coe = coe.substr(0,C_MAXPOWER);
      while(coe.length && (coe[coe.length-1]=='0' || coe[coe.length-1]=='.'))
      {
        coe=coe.substr(0,coe.length-1);
      }
    }
    
    return coe+'e'+exp;    
  }
  
  this['number'] = function(event)
  {
    var dig = parseInt(event.target.getAttribute('key'));
    
    inputNumber(dig);
  }
  function inputNumber(dig)
  {
    if (value_entered.length < C_MAXPOWER)
    {
        value_entered += dig;
        
        // trim leading zeros
        if (value_entered.length > 1 && value_entered[0]=='0' && value_entered[1]!='.')
        {
          value_entered = value_entered.substring(1,value_entered.length);
        }
        
        updateDisplay(value_entered);   
    }
  }

  this['operation'] = function(event)
  {
    if (value_entered)
    {
      self['equals']();
    }
    operation = event.target.getAttribute('key');
    displayOperation(operation);
    value_submitted = value_calculated;
  }

  this['clear-memory'] = function(event)
  {
    memory = 0;
  }
  
  this['recal-memory'] = function(event)
  {
    self['clear']();
    value_calculated = memory;
    updateDisplay(value_calculated);
  }
  
  this['set-memory'] = function(event)
  {
    memory = value_onscreen;
    value_submitted = strToNum(value_entered);
    value_entered = '';
    if (window.widget)
    {
      widget.setPreferenceForKey(memory, 'memory');
    }
  }
  
  this['memory-plus'] = function(event)
  {
    memory += value_onscreen;
    value_submitted = strToNum(value_entered);
    value_entered = '';
    if (window.widget)
    {
      widget.setPreferenceForKey(memory, 'memory');
    }
  }

  this['root'] = function(event)
  {
    var num = 0;
    if( value_entered )
    {
      num = strToNum(value_entered);
      if (num < 0)
      {
        self['clear']();
        updateDisplay(null); // error
      }
      else
      {
        updateDisplay( value_entered = Math.sqrt(num) );
      }
    }
    else
    {
      if (value_calculated < 0)
      {
        self['clear']();
        updateDisplay(null); // error
      }
      else
      {
        value_calculated = Math.sqrt(value_calculated);
        value_submitted = value_calculated;
        updateDisplay( value_calculated );
      }
    }
  }
  
  this['plus-minus'] = function(event)
  {
    if (value_entered)
    {
      if (value_entered[0] == '-')
      {
        value_entered = value_entered.substring(1, value_entered.length);
      }
      else
      {
        value_entered = '-' + value_entered;
      }
      updateDisplay(value_entered);
    }
    else
    {
      value_calculated *= -1;
      updateDisplay(value_calculated);
    }
  }

  this['clear-display'] = function(event)
  {
    value_entered = '';
    value_submitted = 0;
    updateDisplay(0);
  }
  
  this['clear'] = function(event)
  {
    value_entered = '';
    value_submitted = 0;
    value_calculated = 0;
    operation = operation_previous = '';
    displayOperation('')
    updateDisplay(0);
  }  
  
  this['point'] = function(event)
  {
    if (value_entered.indexOf('.') < 0)
    {
      value_entered += (value_entered ? '.' : '0.');
      updateDisplay(value_entered);
    }
  }
  
  this['equals'] = function(event)
  {
    if (value_entered)
    {
      value_submitted = strToNum(value_entered);
    }
    else
    {
      if (!operation && operation_previous)
      {
        operation = operation_previous;
      }
    }
    
    if (operation)
    {
      try
      {
        operations[operation](); // alters value_calculated
      }
      catch (e)
      {
        self['clear']();
        updateDisplay(null); // error
        return;
      }
    }
    else
    {
      value_calculated = value_submitted;
    }
    
    value_entered = '';
    
    operation_previous = operation;
    operation = ''
    displayOperation('');
    
    updateDisplay(value_calculated);
  }

  document.addEventListener('load', init, false);
}