/*************************************************************************************************

License terms as defined in the file license.txt, which MUST be distributed unmodified with this file.

**************************************************************************************************/

/*
	Create a forecast page - all the details for a single day
	
	object forecast - a specific day's data from weather feed.
	object units - default units from weather feed.
*/

ForecastItem = function(forecast, units)
{

	// Create a day/night column with details

	var buildColumn = function(isDay, data)
	{
		var box = document.createElement("div");
		box.className = "forecastcolumn";
		
		var heading = document.createElement("h4");
		if (isDay)
			heading.appendChild(document.createTextNode("Day"));
		else
			heading.appendChild(document.createTextNode("Night"));
		box.appendChild(heading);
		
		var icon = document.createElement("img");
		icon.className = "forecasticon";
		icon.src = "weathericons/"+data.icon+".png";
		box.appendChild(icon);
	
		var link = box.appendChild(document.createElement("a"));
		link.href = forecast.link;
		var text = link.appendChild(document.createElement("h5"));
		var list = box.appendChild(document.createElement("ul"));
		list.className = "compact";
		var temp = list.appendChild(document.createElement("li"));
		var wind = list.appendChild(document.createElement("li"));
		var gust = list.appendChild(document.createElement("li"));
		
		text.appendChild(document.createTextNode(data.text));
		var tempIcon = temp.appendChild(document.createElement("img"));
		tempIcon.className = "dataicon";
		tempIcon.src = "dataicons/ctemp.png";
		var windIcon = wind.appendChild(document.createElement("img"));
		windIcon.className = "dataicon";
		windIcon.src = "dataicons/cwind.png";
		var gustIcon = gust.appendChild(document.createElement("img"));
		gustIcon.className = "dataicon";
		gustIcon.src = "dataicons/cgust.png";

		if (isDay)
			temp.appendChild(document.createTextNode("High " + data.highTemperature + units.temperature));
		else
			temp.appendChild(document.createTextNode("Low " + data.lowTemperature + units.temperature));
		
		wind.appendChild(document.createTextNode(data.windDirection + " at " + data.windSpeed + units.speed));
		gust.appendChild(document.createTextNode("Gusts of " + data.windGusts + units.speed));
	
		return box;
	}
	
	// Create the page

	var container = document.createElement("div");
	container.className = "sliderpage";
	
	// Create the page headings and day/night columns
	
	var dayLink = document.createElement("a");
	dayLink.href = forecast.link;
	dayLink.appendChild(document.createTextNode(forecast.weekday + " " + forecast.date.formatTemplate("{MMM} {dd}, {yyyy}")));
	var dayHeading = document.createElement("h3");
	dayHeading.appendChild(dayLink);
	container.appendChild(dayHeading);
	
	dayCol = buildColumn(true, forecast.dayTime);
	nightCol = buildColumn(false, forecast.nightTime);
	
	var sun = dayCol.getElementsByTagName("ul")[0].appendChild(document.createElement("li"));
	var sunIcon = sun.appendChild(document.createElement("img"));
	sunIcon.className = "dataicon";
	sunIcon.src = "dataicons/csun.png";
	sun.appendChild(document.createTextNode(forecast.sunRise + " - " + forecast.sunSet));
	
	var moon = nightCol.getElementsByTagName("ul")[0].appendChild(document.createElement("li"));
	var moonIcon = moon.appendChild(document.createElement("img"));
	moonIcon.className = "dataicon";
	moonIcon.src = "moonicons/" + forecast.moonIcon + ".png";
	moon.appendChild(document.createTextNode(forecast.moonText));

	container.appendChild(dayCol);
	container.appendChild(nightCol);
	
	// Append to the DOM container

	this.appendTo = function(newParent)
	{
		newParent.appendChild(container);
	}

}

/*
	The same, but the summary version in table-row format
	
	object forecast - a specific day's data from weather feed.
	object units - default units from weather feed.
*/

ForecastSummaryItem = function(forecast, units)
{
	var row = document.createElement("tr");
	var dateCell = row.appendChild(document.createElement("th"));
	dateCell.className = "date";
	var dayCell1 = row.appendChild(document.createElement("td"));
	dayCell1.className = "temp";
	var dayCell2 = row.appendChild(document.createElement("td"));
	dayCell2.className = "desc";
	var nightCell1 = row.appendChild(document.createElement("td"));
	nightCell1.className = "temp";
	var nightCell2 = row.appendChild(document.createElement("td"));
	nightCell2.className = "desc";
	
	var dateLink = dateCell.appendChild(document.createElement("a"));
	dateLink.href = forecast.link;
	dateLink.appendChild(document.createTextNode(forecast.date.formatTemplate("{DD} {dd}")));
	
	var dayIcon = dayCell1.appendChild(document.createElement("img"));
	dayIcon.src = "weathericons/"+forecast.dayTime.icon+".png";
	dayCell1.appendChild(document.createTextNode("H " + forecast.dayTime.highTemperature + units.temperature));

	dayCell2.appendChild(document.createTextNode(forecast.dayTime.text));
	
	var nightIcon = nightCell1.appendChild(document.createElement("img"));
	nightIcon.src = "weathericons/"+forecast.nightTime.icon+".png";
	nightCell1.appendChild(document.createTextNode("L " + forecast.nightTime.lowTemperature + units.temperature));
		
	nightCell2.appendChild(document.createTextNode(forecast.nightTime.text));
	
	// Append to the DOM container
	
	this.appendTo = function(newParent)
	{
		newParent.appendChild(row);
	}

}
