/*
© Copyright 2006 Opera Software ASA. All rights reserved.
 
This file contains Original Code and/or Contributions to the Original 
Code as defined in the Opera Web Applications License (the “License”). 
You may not use this file except in compliance with the License. Please 
obtain a copy of the License at http://www.opera.com/license/owal/
and read it before using this file. 

The Original Code and all Contributions to the Original Code distributed 
under the License are distributed on an “AS IS” basis, WITHOUT WARRANTY 
OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND OPERA HEREBY DISCLAIMS ALL 
SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT. 

Please see the License for the specific language governing rights and 
limitations under the License. 
*/
var Calendar=function (container, numberOfMonth, withWeekNumbers, european, caption)
{
  var currentMonth=(new Date).getMonth();
  var currentYear=(new Date).getFullYear();
  var container=container;
  var numberOfMonth=numberOfMonth||1;
  var withWeekNumbers=(withWeekNumbers&&true)||false;
  var europeanMode=(european&&true)||false;
  var withCaption=(caption&&true)||false;

  var weekNumberName='';
  var allNameOfWeekDays=[];
  var allNameOfWeekDaysLong=[];
  var allNameOfMonths=[];
  var holidays='';
  var lsCountry='';
  var lsLanguage='';
  var lsStyle='';
  var lsMode='';
  var lsWeekNo='';

  var setLocalizedStrings=function()
  {
    weekNumberName=getLocalizedString('Wno');
    allNameOfWeekDays=
    [
      getLocalizedString('Mo'),
      getLocalizedString('Tu'),
      getLocalizedString('We'),
      getLocalizedString('Th'),
      getLocalizedString('Fr'),
      getLocalizedString('Sa'),
      getLocalizedString('Su')
    ];
    allNameOfWeekDaysLong=
    [
      getLocalizedString('Monday'),
      getLocalizedString('Tuesday'),
      getLocalizedString('Wednesday'),
      getLocalizedString('Thursday'),
      getLocalizedString('Friday'),
      getLocalizedString('Saturday'),
      getLocalizedString('Sunday')
    ];
    allNameOfMonths=
    [
      getLocalizedString('January'),
      getLocalizedString('February'),
      getLocalizedString('March'),
      getLocalizedString('April'),
      getLocalizedString('May'),
      getLocalizedString('June'),
      getLocalizedString('July'),
      getLocalizedString('August'),
      getLocalizedString('September'),
      getLocalizedString('October'),
      getLocalizedString('November'),
      getLocalizedString('December')
    ];
    lsCountry=getLocalizedString('Country');
    lsLanguage=getLocalizedString('Language');
    lsStyle=getLocalizedString('Style');
    lsMode=getLocalizedString('Mode');
    lsWeekNo=getLocalizedString('Week Numbers');
  }
  
  var getWeekNumber=function(date)
  {
    var newYear=new Date(date.getFullYear(), 0,1);
    var correction=1;
    var time=date.getTime();
    if(europeanMode)
    {
      time-=(((date.getDay()+13)%7)*24*60*60*1000);
      var day=newYear.getDay();
      if(day==0||day==5||day==6) correction=0;
    }
    else
    {
      time-=((date.getDay())*24*60*60*1000);
    }
    var weeknumber=Math.ceil((time-newYear.getTime())/(7*24*60*60*1000))+correction;
    if(weeknumber) return weeknumber;
    else return getWeekNumber(new Date(date.getFullYear(), -1, 31));
  }

  var getHoliday=function(date)
  {
    var day=date.getDate().toString();
    var month=(date.getMonth()+1).toString();
    var year=date.getFullYear().toString();
    var dayAndMonth=(day.length==1?'0':'')+day+'.'+(month.length==1?'0':'')+month;
    var dateString=dayAndMonth+'.'+year;
    var dateRep=dayAndMonth+'.rep';
    var pos=0;
    var ret='';
    if((pos=holidays.indexOf(dateString))!=-1) 
    {
      ret=(holidays.slice(pos+10, holidays.indexOf(';',pos)));
      
      ret=getLocalizedString(ret).replace(/^[ \u0009]*/,'').replace(/[ \u0009]*$/,'');
      return getLocalizedString(ret).replace(/ /g,'\u00A0');
    }
    if((pos=holidays.indexOf(dateRep))!=-1)
    { 
      ret=(holidays.slice(pos+9, holidays.indexOf(';',pos))).replace(/^[ \u0009]*/,'').replace(/[ \u0009]*$/,'');
      return getLocalizedString(ret).replace(/ /g,'\u00A0');
    }
    return null;
  }

  this.createMonth=function(date)
  {
    var month=date.getMonth();
    var year=date.getFullYear();
    var i=1;
    var dateRun=new Date(year, month, 1, 0, 0, 0);
    var today=new Date();
    today.setHours(0,0,0);
    today=today.toString();
    var weekday=dateRun.getDay();
    container.add('h3',{className:'month'}, allNameOfMonths[month]);
    container.add('h3',{className:'year'}, year)
    var table=container.add('div').add('table');
    /*
    var headrow=table.add('thead').add('tr');
    headrow.add('th',{colSpan:4}, allNameOfMonths[month]);
    headrow.add('th',(withWeekNumbers?{colSpan:4}:{colSpan:3}), year);
    */
    var tbody=table.add('tbody');
    var row=tbody.add('tr');
    var td=null;
    var captionText='';
    if(withWeekNumbers)
    {
      row.add('td', null, weekNumberName);
    }
    var k=0, name='';
    for(; name=allNameOfWeekDays[k]; k++)
    {
      if(!europeanMode) name=allNameOfWeekDays[(k+6)%7];
      row.add('td', null, name);
    }
    row=tbody.add('tr');
    if(withWeekNumbers)
    {
      row.add('td', {className:'weeknumber'}, getWeekNumber(dateRun));
    }
    if(!europeanMode) weekday=(weekday+1)%7;
    while(i%7!=weekday)
    {
      row.add('td');
      i++;
    }
    var currentDay=1;
    var isCurrentMonth=true;
    var holiday='';
    while(isCurrentMonth)
    {
      td=row.add('td', (dateRun.toString()==today?{className:'today'}:null), currentDay);
      if(dateRun.getDay()==0) td.addClass('holiday');
      if(holiday=getHoliday(dateRun))
      {
        if(td.className.indexOf('holiday')==-1) td.addClass('holiday');
        if(withCaption)
        {
          captionText+=' '+currentDay+'.\u00A0'+holiday+' ';
        }
      }
      dateRun.setDate(++currentDay);
      isCurrentMonth=(dateRun.getMonth()==month);
      if(i%7==0)
      {
        row=tbody.add('tr');
        if(withWeekNumbers && isCurrentMonth)
        {
          row.add('td', {className:'weeknumber'}, getWeekNumber(dateRun));
        }
      }
      if(isCurrentMonth)
      {
        i++;
      }
    }
    while((i++)%7!=0) 
    {
      row.add('td');
    }
    if(withCaption)
    {
      container.add('p', null, captionText);
    }
    

  }
  
  this.current_date = -1;

  this.update=function()
  {
    document.getElementById('day').firstChild.nodeValue=
      allNameOfWeekDays[(new Date().getDay()+6)%7];
    document.getElementById('day').title=
      allNameOfWeekDaysLong[(new Date().getDay()+6)%7];
    document.getElementById('date').firstChild.nodeValue=
      ( this.current_date = new Date().getDate() );
    var i=0;
    container.innerHTML='';
    for( ;i<numberOfMonth; i++)
    {
      container.add('hr');
      this.createMonth(new Date(currentYear,currentMonth+i,01));
    }
    container.add('hr');
  }

  this.next=function()
  {
    currentMonth++;
    this.update();
  }
  this.previous=function()
  {
    currentMonth--;
    this.update();
  }

  this.more=function()
  {
    numberOfMonth++;
    this.update();
  }

  this.less=function()
  {
    if(--numberOfMonth<1) numberOfMonth=1;
    this.update();
  }

  this.setEuropeanMode=function(bol)
  {
    europeanMode=bol;
  }

  this.setMonth=function(month)
  {
    currentMonth=month;
  }

  this.setYear=function(year)
  {
    currentYear=year;
  }

  this.setWeekNumbers=function(bol)
  {
    withWeekNumbers=bol;
  }

  this.setCaption=function(bol)
  {
    withCaption=bol;
  }

  this.setHolidays=function(lan)
  {
    var ops=document.getElementsByTagName('select')[0].options, option=null, i=0;
    for( ; option=ops[i]; i++)
    {
      if(option.value==lan) option.selected=true;
      else option.selected=false;
    }
    setCookie('calendar-holidays', lan);
    var ele=document.getElementById('holidays');
    if(ele) ele.parentNode.removeChild(ele);
    ele=document.getElementsByTagName('head')[0].
      appendChild(document.createElement('script'));
    ele.type='text/javascript';
    ele.id='holidays';
    ele.src='holidays/'+lan+'.js';
    holidays=localizedHolidays;
    this.update();
  }

  this.setLanguage=function(lan)
  {
    var ops=document.getElementsByTagName('select')[1].options, option=null, i=0;
    for( ; option=ops[i]; i++)
    {
      if(option.value==lan) option.selected=true;
      else option.selected=false;
    }
    setCookie('calendar-language', lan);
    var ele=document.getElementById('language');
    if(ele) ele.parentNode.removeChild(ele);
    ele=document.getElementsByTagName('head')[0].
      appendChild(document.createElement('script'));
    ele.type='text/javascript';
    ele.id='language';
    ele.src='languages/'+lan+'.js';
    setLocalizedStrings();
    document.getElementsByTagName('label')[0].firstChild.nodeValue=lsCountry+':';
    document.getElementsByTagName('label')[1].firstChild.nodeValue=lsLanguage+':';
    document.getElementsByTagName('label')[2].firstChild.nodeValue=lsStyle+':';
    document.getElementsByTagName('label')[3].firstChild.nodeValue=lsMode+':';
    document.getElementsByTagName('label')[4].firstChild.nodeValue=lsWeekNo+':';
    this.update();
  }
}

var calendar_2=null;
var year=0;
var Controller=function()
{
  var self=this;
  var calendar=new Calendar(document.getElementById('calendar'), 3, 'withWeekNumbers','europaeisch','withCaption');

  calendar.setMonth(new Date().getMonth());
  calendar.setYear(new Date().getFullYear());
  

  calendar.setLanguage(getCookie('calendar-language')||'eng');

  var numberOfMonthToDisplay=3;
  var numberOfMonthToDisplayed=0;


  var transition_less=document.getElementById('less').createTransition();
  transition_less.styles.opacity=0;
  var transition_more=document.getElementById('more').createTransition();
  transition_more.styles.opacity=1;
  var transition_previous=document.getElementById('previous').createTransition();
  transition_previous.styles.opacity=0;
  var transition_next=document.getElementById('next').createTransition();
  transition_next.styles.opacity=0;
  var transition_info=document.getElementById('infoButton').createTransition();
  transition_info.styles.opacity=0;

  var fadeInfoOut=function()
  {
    if(transition_info.styles.opacity==1)
    {
      transition_info.add('opacity', 1, 0).execute();
    }
  }

  var front=document.getElementById('front');

  front.onmouseover=function()
  {
    clearTimeout(this.timeout);
    if(transition_info.styles.opacity==0)
    {
      transition_info.add('opacity', 0, 1).execute();
    }
  }

  front.onmouseout=function()
  {
    this.timeout=setTimeout(fadeInfoOut, 20);
  }

  var bs=document.getElementById('control').getElementsByTagName('b'), b=null, i=0;
  for( ; b=bs[i]; i++)
  {
    b.trans=b.createTransition();
    b.style.opacity=0;
    b.onmouseover=function()
    {
      if(this.parentNode.style.opacity)
      {
        this.trans.add('opacity', 0, 1).execute();
      }
    }
    b.onmouseout=function()
    {
      this.trans.add('opacity', 1, 0).execute();
    }
  }
  
  var container=document.getElementById('calendar');
  var transition=container.createTransition();

  transition.transform=function(x)
  {
    var sin=Math.sin;
    var pi=Math.PI;
    var start=.2;
    return (sin((x)/100*pi))+start;
  }

  var checkControlles=function()
  {
    switch (numberOfMonthToDisplayed)
    {
    case 0:
      transition_less.add('opacity', 1, 0).execute();
      transition_next.add('opacity', 1, 0).execute();
      transition_previous.add('opacity', 1, 0).execute();
      break;
    case 3:
      if(transition_more.styles.opacity==1)
      {
        transition_more.add('opacity', 1, 0).execute();
      }
      break;
    default:
      var checks=[transition_less, transition_more, transition_next, transition_previous];
      var pointer=null, i=0;
      for( ; pointer=checks[i]; i++)
      {
        if(pointer.styles.opacity==0)
        {
          pointer.add('opacity', 0, 1).execute();
        }
      }
    }
  }

  container.style.height='0px';
  document.getElementById('calendar').style.height=0;

  this.adjustHeight=function()
  {
    var cor=container==container.getElementsByTagName('hr')[0].offsetParent?0:container.offsetTop;
    transition.add
    (
      'height', 
      container.style.height,
      (container.getElementsByTagName('hr')[numberOfMonthToDisplayed].offsetTop-cor)+'px'
    );
    transition.execute();
  }

  this.calenderplus=function() 
  {
    if(!transition.checkRun())
    {
      calendar.next();
      this.adjustHeight();
    }
  }

  this.calenderminus=function() 
  {
    if(!transition.checkRun())
    {
      calendar.previous();
      this.adjustHeight();
    }
  }

  this.calendermore=function() 
  {
    if(!transition.checkRun() && numberOfMonthToDisplayed<numberOfMonthToDisplay)
    { 
      numberOfMonthToDisplayed++;
      setCookie('calendar-month', numberOfMonthToDisplayed);
      this.adjustHeight();
    }
  }

  this.calenderless=function() 
  {
    if(!transition.checkRun())
    {
      if(--numberOfMonthToDisplayed<0) numberOfMonthToDisplayed=0;
      setCookie('calendar-month', numberOfMonthToDisplayed);
      this.adjustHeight();
    }
  }
  
  var checkDay = function() 
  {
    if( calendar.current_date != new Date().getDate() )
    {
      if(!transition.checkRun())
      {
        calendar.update();
        self.adjustHeight();
      }
    }
  }

  this.setMode=function(mode)
  {
    var ops=document.getElementsByTagName('select')[3].options, option=null, i=0;
    for( ; option=ops[i]; i++)
    {
      if(option.value==mode) option.selected=true;
      else option.selected=false;
    }
    setCookie('calendar-mode', mode);
    calendar.setEuropeanMode(mode=='eu');
    calendar.update();
  }

  this.setWithWeekNumbers=function(bol)
  {
    document.getElementsByTagName('input')[0].checked=bol;
    calendar.setWeekNumbers(bol);
    setCookie('calendar-withWeeks', bol.toString());
    calendar.update();
  }

  this.setStyle=function(style)
  {
    var ops=document.getElementsByTagName('select')[2].options, option=null, i=0;
    for( ; option=ops[i]; i++)
    {
      
      if(option.value==style) option.selected=true;
      else option.selected=false;
    }
    setCookie('calendar-style', style);
    var ele=document.getElementById('style');
    if(ele) ele.parentNode.removeChild(ele);
    ele=document.getElementsByTagName('head')[0].
      appendChild(document.createElement('link'));
    ele.rel='stylesheet';
    ele.id='style';
    ele.href='styles/'+style+'/style.css';
    calendar.update();
  }

  this.setLanguage=function(lang)
  {
    calendar.setLanguage(lang);
  }
  this.setHolidays=function(lang)
  {
    calendar.setHolidays(lang);
  }
  this.setLanguage(getCookie('calendar-language')||'eng');
  this.setStyle(getCookie('calendar-style')||'metal');
  this.setMode(getCookie('calendar-mode')||'eu');
  this.setHolidays(getCookie('calendar-holidays')||'ch');
  this.setWithWeekNumbers(getCookie('calendar-withWeeks')?getCookie('calendar-withWeeks')=='true':true);
  var monthStored=getCookie('calendar-month', numberOfMonthToDisplayed)||1;
  monthStored=parseInt(monthStored);
  var checkStartup=function()
  {
    if(numberOfMonthToDisplayed<monthStored)
    {
      self.calendermore();
      checkControlles();
    }
    else
    {
      transition.callback=checkControlles;
    }
  }
  transition.callback=checkStartup;
  checkStartup();
  setInterval(checkDay, 60*1000 );
}

var controller=null;

onload=function()
{
  controller=new Controller();
}
