if (!window.widget) { window.widget = {}; window.widget.openURL = function(u) { window.open(u) } }

g_feeds = {};
g_feedUrlBase = "http://oxine.opera.com/~runeh/feeds/"
g_activeFeed = null;
g_lastFeed = null; // lastFeed to work properly
g_reloadFeedInterval = 0;


C_OUTDATED_TIMEOUT = 15*60*1000; // how long (milliseconds) until a source is considered out-dated

window.onload = init; // init is at the bottom of this script

/*
 A preference is set for the last used feed
 called "lastFeed". This is the id of the
 feed last displayed.
 */

function Feed(props)
{
    var self = this;
    
    var url = g_feedUrlBase + props.id + ".xml";
    
    var active = false; // requesting or not

    var items = null; // js-object of results
    
    this.props = props;
    
    var failSilent = false;
    
    // if silent then a fail will not tell the user
    this.show = function()
    {
        failSilent = false;
        
        // update every now and again
        if (g_reloadFeedInterval)
        {
            clearInterval(g_reloadFeedInterval);
        }
        g_reloadFeedInterval = setInterval(function()
        {
            failSilent = true;
            items = null;
            self.load();
        }, C_OUTDATED_TIMEOUT);
        
        if (items)
        {
            var timePassed = (new Date()).getTime() - items.loadTime;
            if (timePassed < 0 || timePassed > C_OUTDATED_TIMEOUT)
            {
                items = null;
            }
        }
        if (items)
        {
            g_lastFeed = this.props.id;
            showLoadingState('ok');
            setTimeout(function(){renderNewsItems(items)},200);
            widget.setPreferenceForKey(props.id, 'lastFeed');
        }
        else
        {
            showLoadingState('loading');
            this.load();
        }
    };
    this.load = function(silent)
    {
        if (!active)
        {
            active = true;
            getXML(url, loadCallback);
        }
    };
    function loadCallback(xml)
    {
        active = false;
        var results = xml && parseFeed(xml);
        var focused = (g_feeds[g_activeFeed] == self); // whether or not the feed is focused
        if (results)
        {
            // store results
            items = results;
            // show us if we are selected
            if (focused)
            {
                self.show();
            }
        }
        else
        {
            if (focused && !failSilent)
            {
                showLoadingState('fail');
            }
        }
    }
}

function loadPreviousFeed()
{
    if (g_lastFeed)
    {
        selectSource(g_lastFeed);
    }
    else
    {
        selectSource();
        document.getElementById('newslist').innerHTML = '<li>Select a news source on the left hand side</li>';
        showLoadingState('ok');
    }
}

document.onkeypress = function(e)
{
  var ele = document.getElementById('newsbox');
  var off = ele.scrollTop;
  
  if (e.keyCode == 38)
  {
    off -= 40;
  }
  else if (e.keyCode == 40)
  {
    off += 40;
  }
  else { return; }

  ele.scrollTop = Math.max(0, Math.min(ele.scrollHeight-ele.offsetHeight, off));
  e.preventDefault();
}


/*
    Request a file with url and callback
    Callback gets the xml or null
    Returns an abortion function
*/
function getXML(url, callback)
{
    var xhr = null;
    
    var timeout = setTimeout(function()
    {
        if (xhr && xhr.abort)
        {
            timeout = 0;
            xhr.abort();
            callback(null);
        }
    }, 60*1000); // 60 second max timer
    
    try {
        xhr = new XMLHttpRequest();
        xhr.onreadystatechange = function()
        {
            if (timeout && this.readyState == 4) {
                clearTimeout(timeout);
                timeout = 0;
                callback(this.status == 200 ? this.responseXML : null);
            }
        }
        xhr.open('GET',url,true);
        xhr.send();
    }
    catch (e)
    {
        clearTimeout(timeout);
        timeout = 0;
        // avoid synchronous callback
        setTimeout(function()
        {
            callback(null);
        },1);
    }
    
    return function() // returns an abort function
    {
        if (timeout && xhr && xhr.abort)
        {
            clearTimeout(timeout);
            timeout = 0;
            xhr.abort();
            return true;
        }
        return false;
    };
};



// showLoadingState();

(function(){
    var interval = 0;
    var frameCount = 0;
    
    var ele = null; // container
    var ani = null; // animating div
    
    window.showLoadingState = function(state, callback)
    {
        if (!ele) { ele = document.getElementById('loadingbox'); }
        
        switch (state)
        {
            case 'loading' :
                ele.innerHTML = '<div></div><button>Cancel</button>';
                ani = ele.getElementsByTagName('div')[0];
                ele.getElementsByTagName('button')[0].onclick = loadPreviousFeed;
                ele.style.display = 'block';
                
                if (!interval) { interval = setInterval(frame, 35); }
                break;
            
            case 'list' : // getting the list                
                ele.innerHTML = '<div></div>Loading Feeds';
                ani = ele.getElementsByTagName('div')[0];
                ele.style.display = 'block';
                
                if (!interval) { interval = setInterval(frame, 35); }
                break;
            
            case 'list-fail' : // failed getting list
                ele.innerHTML = '<img src="img/nonet.png"><span>No network available</span><p>Please exist the widget and enable a connection.</p>';
                /*
                 <button>Retry</button>
                 ele.getElementsByTagName('button')[0].onclick = function()
                {
                    showLoadingState('list');
                    onRefreshInterval();
                }
                */
                ele.style.display = 'block';
                break;
            
            case 'ok' :
                ele.innerHTML = '';
                ele.style.display = ''; // none
                clearInterval(interval);
                interval = 0;
                break;
            
            case 'fail' :
                ele.innerHTML = '<img src="img/nonet.png"><span>No network available</span><button>OK</button>'; 
                ele.getElementsByTagName('button')[0].onclick = loadPreviousFeed;
                ele.style.display = 'block';
                break;
            
            default :
                opera.postError('Unrecognised state: '+state);
        }
    }
    function frame()
    {
        frameCount++;
        ani.style.backgroundPosition = ((-frameCount*100) %1200) + 'px';
    }
})();



function renderNewsItems(items) {

	var container = document.getElementById("newslist");
	while (container.firstChild) { container.removeChild(container.firstChild) };
	container.scrollTop = 0;
	var frag = document.createDocumentFragment();
	
    var li = null;
    var h2 = null;
    var p = null;
    var img = null;
    var a = null;
    var span = null;
    
    /*  Example markup:
        <li>
            <span>31 october 1999</span>
            <h2>Man dies by heart failure!!!11</h2>
            <p>
                <img src="a.jpg">
                Yes it's true, last night some guy
                died...
                <a>Read more...</a>
            </p>
        </li>
    */
    
	for (var n=0, item; item=items[n]; n++) {
        
		li = document.createElement("li");
        li.url = item.url;
		li.addEventListener("click", openNewsItem, false);
        
        span = document.createElement("span");
		span.textContent = formatTimeStamp(new Date(item.timestamp));
        li.appendChild(span);
        
		h2 = document.createElement("h2");
		h2.textContent = item.title;
        li.appendChild(h2);

        p = document.createElement("p");        
		{
            if (item.img) {
                img = document.createElement("img");
                img.src = item.img;
                p.appendChild(img);
            }
            
            p.appendChild(document.createTextNode(item.blurb));
            
            a = document.createElement("a");
            a.textContent = "Read more..";
            p.appendChild(a);
        }
		li.appendChild(p);

		frag.appendChild(li);
	}
	container.appendChild(frag);
}

function formatTimeStamp(d) {
	var a = [zp(d.getDay()), ".", zp(d.getMonth()), ", ", zp(d.getHours()), ":", zp(d.getMinutes())];
	return a.join("");
}

// zero pad
function zp(n) {
	return ("00"+n).slice(-2);
}

function openNewsItem(evt) {
	ele = evt.srcElement;
	var url = "";
	while(!(url = ele.url)) { ele = ele.parentNode }
	widget.openURL(url);
}

function selectSource(evt_or_id) {
	var id = evt_or_id;
	if (evt_or_id && typeof evt_or_id != "string") {
		ele = evt_or_id.srcElement;
		while(!(id = ele.sourceId)) { ele = ele.parentNode }
	}

	var sources = document.getElementById("sourcelist").getElementsByTagName("li");
	for (var n=0, e; e=sources[n]; n++) {
		if (e.sourceId == id) {
			e.className = "active_feed";
		} else if (e.className!="") {
			e.className = "";
		}
	}
	
	g_activeFeed = id;
    if (g_feeds[id]){
        g_feeds[id].show();
    }
}

function renderNewsSources(sourcedict) {
	var container = document.getElementById("sourcelist");
	while (container.firstChild) { container.removeChild(container.firstChild) };
	
    var lastFeed = widget.preferenceForKey('lastFeed');
    var defaultFeed = null;
    
	for (id in sourcedict) {
		var source = sourcedict[id].props;
		var li = document.createElement("li");
		li.sourceId = source.id;
		var icon = document.createElement("img");
		icon.setAttribute("src", source.icon);
		li.addEventListener("click", selectSource, false);
		li.appendChild(icon);
		li.appendChild(document.createTextNode(source.title));		
		container.appendChild(li);
        if (!defaultFeed || lastFeed == source.id)
        {
            defaultFeed = source.id;
        }
	}
    
    selectSource(defaultFeed);
}

function getListCallback(xml) {
	if (xml) {
        parseFeedData(xml);
	}
    else {
        showLoadingState('list-fail');
	}
}

function parseFeedData(xml) {
	if (xml.documentElement && xml.documentElement.nodeName=="feedlist") {
		parseFeedList(xml);
		renderNewsSources(g_feeds);
	}
}

function parseFeedList(xml) {
	var feeds = xml.getElementsByTagName("feed");	
	for (var n=0, feed; feed=feeds[n]; n++) {
		var cur = {};
		cur.title = feed.getAttribute("title");
		cur.updated = feed.getAttribute("updated");
		cur.id = feed.getAttribute("id");
		cur.icon = feed.getAttribute("icon") || "img/icon_rss.png";
        cur.url = 
		g_feeds[cur.id] = new Feed(cur);
	}
}

/*
 takes the xml
 returns the items
*/

function parseFeed(xml) {
    
    try {
        var elems = xml.getElementsByTagName("entry");
        
        var items = [];
        
        for (var n=0, item; item=elems[n]; n++) {
            var cur = {};
            cur.title = item.getAttribute("title");
            cur.timestamp = +(item.getAttribute("timestamp"));
            cur.url = decodeURI(item.getAttribute("url"));
            if (item.getAttribute("img")) {
                cur.img = decodeURI(item.getAttribute("img"));
            }
            cur.blurb = item.textContent || "No summary";
            items.push(cur);
        }
        
        var id = xml.documentElement.getAttribute("id");
    
    
        items.sort(function(a,b) {
                                    if (a.timestamp>b.timestamp) {return 1}
                                    else if (a.timestamp<b.timestamp) { return -1 }
                                    else {return 0}
                                }
                            );
        items.reverse();
        items = items.slice(0,10);
        items.loadTime = (new Date()).getTime();
    }
    catch (e)
    {
        return null;
    }

    return items;
}

function onRefreshInterval() {
	getXML(g_feedUrlBase + "feeds.xml", getListCallback);
}

function init() {
    showLoadingState('list');
	onRefreshInterval();
}
