/**
 * Copyright (c) 2007, Opera Software ASA
 * All rights reserved.
 */

/**
 * Simple TEA implementation
 *
 * Throws exception for input characters above 0xFF
 *
 * @author Magnus Kristansen
 * @requires sha1.js hex_sha1(str)
 */
var UINT32_MAX = 0x100000000;

function hex_word(str) {
	var ret = [];
	for ( var i = 0; i < str.length; i+=8 ) {
		ret[i>>3] = + ( '0x' + str.slice( i, i+8 ) );
	}
	//opera.postError( 'h_w In: ' + str + '\n' + 'Out: ' + ret );
	return ret;
}
function word_hex(ar) {
	var ret = [];
	for ( var i = 0; i < ar.length; i++ ) {
		ret[i] = ( UINT32_MAX + ar[i] ).toString(16).slice(-8);
	}
	//opera.postError( 'w_h In: ' + ar + '\n' + 'Out: ' + ret );
	ret = ret.join('');
	return ret;
}
function str_word(str) {
	if ( /[^\x00-\xFF]/.test(str) ) { throw 'tea_code: Illegal character in string'; }
	if ( str.length % 2 ) str += '\0';
	var ret = [];
	for ( var i = 0; i < str.length; i+=2 ) {
		ret.push( str.charCodeAt(i) * 0x100 + str.charCodeAt(i+1) );
	}
	//opera.postError( 's_w In: ' + str + '\n' + 'Out: ' + ret );
	return ret;
}
function word_str(ar) {
	var ret = [];
	for ( var i = 0; i < ar.length; i++ ) {
		ret[i] = String.fromCharCode( ar[i] >> 8, ar[i] % 0x100 );
	}
	//opera.postError( 'w_s In: ' + ar + '\n' + 'Out: ' + ret );
	ret = ret.join('');
	if ( ret.slice(-1) == '\0' ) ret = ret.slice( 0, -1 );
	return ret;
}
function uint32_random() {
	return Math.floor( Math.random() * UINT32_MAX );
}
function uint32_box(n) {
	n = n % UINT32_MAX;
	if ( n < 0 ) n += UINT32_MAX;
	return n;
}
function tea_ecb(value,key,enc) {
	var k = hex_word( hex_sha1( key ) );

	if ( enc ) {
		var v_in = str_word( value );

		var v_out = [];
		for ( var i = 0; i < v_in.length; i+=2 ) {
			var v_temp = v_in.slice( i, i+2 );
			tea_raw_encode( v_temp, k );
			v_out.push( v_temp[0], v_temp[1] );
		}
		return word_hex( v_out );
	} else {
		var v_in = hex_word( value );

		var v_out = [];
		for ( var i = 0; i < v_in.length; i+=2 ) {
			var v_temp = v_in.slice( i, i+2 );
			tea_raw_decode( v_temp, k );
			v_out.push( v_temp[0], v_temp[1] );
		}
		return word_str( v_out );
	}
}
function tea_ofb(value,key,enc) {
	var k = hex_word( hex_sha1( key ) );

	if ( enc ) {
		var iv = [ uint32_random(), uint32_random() ];
		var iv_hex = word_hex( iv );
		var v = str_word( value );
	} else {
		value = value.split('-');
		var iv = hex_word( value[0] );
		var v = hex_word( value[1] );
	}

	for ( var i = 0; i < v.length; i+=2 ) {
		tea_raw_encode( iv, k );
		v[i] = uint32_box( v[i] ^ iv[0] );
		v[i+1] = uint32_box( v[i+1] ^ iv[1] );
	}

	if ( enc ) {
		return iv_hex + '-' + word_hex( v );
	} else {
		return word_str( v );
	}
}
/*
void code(long* v, long* k)  {
unsigned long y=v[0],z=v[1], sum=0,
 delta=0x9e3779b9, n=32 ;
while (n-->0) {
  sum += delta ;
    y += (z<<4)+k[0] ^ z+sum ^ (z>>5)+k[1] ;
    z += (y<<4)+k[2] ^ y+sum ^ (y>>5)+k[3] ;
              }
v[0]=y ; v[1]=z ; }
*/
function tea_raw_encode(v,k) {
	var y = v[0];
	var z = v[1];
	var delta = 0x9e3779b9;
	var sum = 0;
	var n = 32;
	while ( n-- ) {
		sum += delta;
		y += ( (z<<4) + k[0] ) ^ ( z + sum ) ^ ( (z>>5) + k[1] );
		z += ( (y<<4) + k[2] ) ^ ( y + sum ) ^ ( (y>>5) + k[3] );
	}
	v[0] = uint32_box( y );
	v[1] = uint32_box( z );
}
/*
void decode(long* v,long* k)  {
 unsigned long n=32, sum, y=v[0], z=v[1],
 delta=0x9e3779b9 ;
sum=delta<<5 ;
while (n-->0) {
    z-= (y<<4)+k[2] ^ y+sum ^ (y>>5)+k[3] ;
    y-= (z<<4)+k[0] ^ z+sum ^ (z>>5)+k[1] ;
   sum-=delta ;  }
v[0]=y ; v[1]=z ;  }
*/
function tea_raw_decode(v,k) {
	var y = v[0];
	var z = v[1];
	var delta = 0x9e3779b9;
	var sum = delta<<5;
	var n = 32;
	while ( n-- ) {
		z -= ( (y<<4) + k[2] ) ^ ( y + sum ) ^ ( (y>>5) + k[3] );
		y -= ( (z<<4) + k[0] ) ^ ( z + sum ) ^ ( (z>>5) + k[1] );
		sum -= delta;
	}
	v[0] = uint32_box( y );
	v[1] = uint32_box( z );
}
